class Log {
  constructor () {
    this.debug = true;
  }

  Logcat (...args) {
    if (this.debug) {
      console.log.call(console, `${new Date().toLocaleString()}anyRTC SDK [DEFAULT]: `, args);
    }
  }
}

class EventEmitter extends Log {
    constructor () {
        super();
        this.events = {};
    }

    //绑定事件函数
    on (eventName, callback) {
        this.events[eventName] = this.events[eventName] || [];
        this.events[eventName].push(callback);
    };
    //触发事件函数
    emit (eventName, _) {
        var events = this.events[eventName],
            args = Array.prototype.slice.call(arguments, 1),
            i, m;

        if (!events) {
            return;
        }
        for (i = 0, m = events.length; i < m; i++) {
            events[i].apply(null, args);
        }
    }
}

class RTMPCXcxKit extends EventEmitter {
    constructor () {
        super();
        let that = this;

        this.url = 'ws://teameeting.anyrtc.io:9095';
        this.webSocket = null;

        //所在房间
        this.anyrtcId = '';
        this.applyLine = false;
        //用户配置
        this.userId = '';
        this.userName = '';
        this.userData = '';
        //系统帐号配置
        this.devId = "";
        this.appId = "";
        this.appKey = "";
        this.appToken = "";
        //自己的ID，由后服务器创建
        this.my_id = "";
        this.seqn = 0;

        this.timer = null;
    }

    // 字节计算
    strlen(str) {
        var len = 0;
        for (var i = 0; i < str.length; i++) {
            var c = str.charCodeAt(i);
            //单字节加1
            if ((c >= 0x0001 && c <= 0x007e) || (0xff60 <= c && c <= 0xff9f)) {
                len++;
            }
            else {
                len += 2;
            }
        }
        return len;
    }

    doInitWSocket () {
        let that = this;

        if (that.webSocket == null) {
            var strUrl = that.url + "/?Type=live_g_wx&UserId=" + that.devId + "&AppId=" + that.appId + "&AppKey=" + that.appKey + "&Token=" + that.appToken;

            that.webSocket = wx.connectSocket({
                url: strUrl,
                success: function (res) {
                    that.Logcat('connectSocket success', res)
                },
                fail: function (error) {
                    that.Logcat(error)
                }
            });

            that.webSocket.onOpen(function (data) {
                that.Logcat('onOpen', data);
            });

            that.webSocket.onMessage(function (evt) {
                var received_msg = evt.data;
                that.Logcat("RecvMsg: " + received_msg);

                var jsResp = JSON.parse(received_msg);
                if (jsResp.Cmd != undefined) {
                    if (jsResp.Code == 200) {
                        that.my_id = jsResp.DyncID;

                        var objMsg = {};
                        objMsg["Cmd"] = "Join";
                        objMsg["AnyrtcId"] = that.anyrtcId;
                        objMsg["UserId"] = that.userId;
                        objMsg["UserName"] = that.userName;
                        objMsg["UserData"] = that.userData;
                        that.Logcat(objMsg)
                        that.doSendMessage(JSON.stringify(objMsg));

                        that.doKeepyalive();
                    }
                }
                else if (jsResp.Msgs != undefined) {
                    var jsMsgs = jsResp.Msgs;
                    for (var i = 0; i < jsMsgs.length; i++) {
                        that.Logcat(jsMsgs[i]);
                        var jMsg = JSON.parse(jsMsgs[i]);
                        
                        if (jMsg.Seqn <= that.seqn) {
                            continue;
                        }
                        that.seqn = jMsg.Seqn;
                        /** Liveing
                         DC_JOIN_LIVE = 4001,
                         DC_NOTIFY_LIVE,
                         DC_LEAVE_LIVE,
                         */
                        if (jMsg.Cmd == 4001) {// DC_JOIN_LIVE
                            if (jMsg.Params.Result == "ok") {
                                var jsBody = JSON.parse(jMsg.Body);

                                /**
                                 *  RTC服务连接结果
                                 *  @params nCode       0 成功
                                 **/
                                that.emit("onRTCJoinLineResult", 0);
                            }
                            else {
                                that.my_id = "";
                                that.isAudioLive = false;
                                that.webSocket = null;
                                /**
                                 *  RTC服务连接结果
                                 *  @params nCode       605     RTCLive_IS_STOP
                                 **/
                                that.emit("onRTCJoinLineResult", 605);
                            }
                        }
                        else if (jMsg.Cmd == 4002) {// DC_NOTIFY_LIVE
                            var jsBody = JSON.parse(jMsg.Body);
                            if (jsBody.CMD == "AcceptApply") {//*Guest
                                /**
                                1, 取消拉流
                                2，进入房间
                                */

                                /**
                                 *  主播接收游客连麦
                                 *  @params strLivePeerId       连麦者标识id（用于标识连麦用户，每次连麦随机生成）
                                 **/
                                that.emit("onRTCApplyLineResult", 0, jsBody.LivePeerID);
                            }
                            else if (jsBody.CMD == "RejectApply") {//*Guest
                                that.applyLine = false;
                                /**
                                 *  主播拒绝游客连麦
                                 *
                                 **/
                                that.emit("onRTCApplyLineResult", jsBody.Code ? jsBody.Code : 601);
                            }
                            else if (jsBody.CMD == "HangupLine") {//*Guest
                                /**
                                1, 离开房间
                                2，继续拉流
                                */

                                that.applyLine = false;
                                /**
                                 *  主播挂断游客连麦
                                 **/
                                that.emit("onRTCHangupLine");
                            }
                            else if (jsBody.CMD == "JoinXcx") {//
                                var RoomID = jsBody.RoomID;
                                var UserID = jsBody.UserID;
                                var XcxAppID = jsBody.XcxAppID;
                                var UserSig = jsBody.UserSig;

                                that.getRoomSig(RoomID, UserID, XcxAppID, UserSig);
                            }
                            else if (jsBody.CMD == "UserMsg") {
                                /**
                                 *  收到消息回调
                                 *  @params nType               消息类型    0 普通消息  1 弹幕消息
                                 *  @params strUserId           游客在开发者平台的userid     jsBody.UserName
                                 *  @params strUserName         游客在开发者平台的用户昵称
                                 *  @params strUserHeaderUrl    游客在开发者平台的用户头像
                                 *  @params strMessage          游客申请连麦时带过来的自定义参数体（可查看游客端申请连麦参数）
                                 **/
                                that.emit("onRTCUserMessage", 0, jsBody.UserName, jsBody.NickName, (jsBody.HeaderUrl == undefined ? "" : jsBody.HeaderUrl), jsBody.Content);
                            }
                            else if (jsBody.CMD == "UserBarrage") {
                                /**
                                 *  收到消息回调
                                 *  @params nType               消息类型    0 普通消息  1 弹幕消息
                                 *  @params strUserId           游客在开发者平台的userid
                                 *  @params strUserName         游客在开发者平台的userid
                                 *  @params strUserHeaderUrl    游客在开发者平台的userid
                                 *  @params strMessage          游客申请连麦时带过来的自定义参数体（可查看游客端申请连麦参数）
                                 **/
                                that.emit("onRTCUserMessage", 1, jsBody.UserName, jsBody.NickName, (jsBody.HeaderUrl == undefined ? "" : jsBody.HeaderUrl), jsBody.Content);
                            }
                            else if (jsBody.CMD == "MemberNotify") {
                                var nTotal = jsBody.Total,
                                    RoomID = jsBody.RoomID,
                                    RoomSvrID = jsBody.ServerID;
                                /**
                                 *  直播间实时在线人数变化通知
                                 *  @params nTotalMember        当前活动在线实时人数
                                 **/
                                that.emit("onRTCMemberListNotify", nTotal, RoomID, RoomSvrID);
                            }
                            else if (jsBody.CMD == "LiveStart") {//*Guest
                                /**
                                 *
                                 **/
                                that.emit("onRTCLiveStart");    // 直播开始
                            }
                            else if (jsBody.CMD == "LiveStop") {//*Guest
                                that.applyLine = false;
                                /**
                                 *
                                 **/
                                that.emit("onRTCLiveStop"); // 主播暂时离开
                            }
                        }
                        else if (jMsg.Cmd == 4003) {// DC_LEAVE_LIVE

                            if (jMsg.Body == "Live is over!") {
                                /**
                                 *
                                 **/
                                that.emit("onRTCLineLeave");    // 主播关闭活动
                            }
                        }
                    }
                }
            });

            that.webSocket.onClose = function () {
                that.Logcat('webSocket onClose')
                that.webSocket = null;
            };
        }
    }

    doKeepyalive  () {
        let that = this;
        
        if (that.my_id != "") {
            that.doSync();
            that.timer && clearTimeout(that.timer);
            that.timer = setTimeout(()=>{
                that.doKeepyalive();
            }, 10000);
        }
    }
    
    doSync () {
        let that = this;

        var objMsg = {};
        objMsg["Cmd"] = "Sync";
        objMsg["ClintSeqn"] = that.seqn;
        that.doSendMessage(JSON.stringify(objMsg));
    }

    //发送指令
    doSendMessage (strMsg) {
        let that = this;

        if (that.webSocket != null) {
            that.webSocket.send({
                data: strMsg,
                complete: res => {
                    that.Logcat(res);
                }
            });
        }
    }
    
    //断开链接
    doUnInitWSocket () {
        let that = this;

        if (that.webSocket != null) {
            that.webSocket.close();
            that.webSocket = null;
        }
    }

    getRoomSig(roomID, pubID, sdkAppID, userSig) {
        let that = this;

        let url = 'https://official.opensso.tencent-cloud.com/v4/openim/jsonvideoapp';
        url += '?sdkappid=' + sdkAppID + "&identifier=" + pubID + "&usersig=" + userSig + "&random=9999&contenttype=json";

        let reqHead = {
            "Cmd": 1,
            "SeqNo": 1,
            "BusType": 7,
            "GroupId": parseInt(roomID)
        };
        let reqBody = {
            "PrivMapEncrypt": "",
            "TerminalType": 1,
            "FromType": 3,
            "SdkVersion": 26280566
        };

        wx.request({
            url: url,
            data: {
                "ReqHead": reqHead,
                "ReqBody": reqBody
            },
            method: "POST",
            success: function (res) {
                that.Logcat("requestSigServer success:", res);

                let roomSig = JSON.stringify(res.data["RspBody"]);
                let pushUrl = "room://cloud.tencent.com?sdkappid=" + sdkAppID + "&roomid=" + roomID + "&userid=" + pubID + "&roomsig=" + encodeURIComponent(roomSig);
                that.Logcat("roomSigInfo", roomID, pubID, roomSig, pushUrl);

                that.emit("onGetPushUrl", 0, {
                    pushURL: pushUrl,
                    pubID: pubID
                });
            },
            fail: function (res) {
                that.emit("onGetPushUrl", -1, res);
            }
        })
    }

    NotifyError (methodName, errMsg) {
        let that = this;
        /**
        * @errorMethodName   错误的方法名称
        * @errorMethodMsg    错误消息提示
        */
        that.emit('onSDKError', methodName, {
            msg: errMsg
        });
    }

    /**
         *  配置开发者信息
         *  @params strDeveloperId      anyRTC 云平台的开发者id
         *  @params strAppId            anyRTC云平台的应用id
         *  @params strAppKey           anyRTC云平台的应用的appKey
         *  @params strAppToken         anyRTC云平台的应用的appToken
         *  API说明                      配置anyRTC云平台开发者信息。
         **/
    initEngineWithAnyRTCInfo (strDeveloperId, strAppId, strAppKey, strAppToken) {
        var that = this;

        if (typeof strDeveloperId !== "string") {
            that.NotifyError('initEngineWithAnyRTCInfo', 'type of strDeveloperId is not string.');
            return false;
        }
        if (typeof strAppId !== "string") {
            that.NotifyError('initEngineWithAnyRTCInfo', 'type of strAppId is not string.');
            return false;
        }
        if (typeof strAppKey !== "string") {
            that.NotifyError('initEngineWithAnyRTCInfo', 'type of strAppKey is not string.');
            return false;
        }
        if (typeof strAppToken !== "string") {
            that.NotifyError('initEngineWithAnyRTCInfo', 'type of strAppToken is not string.');
            return false;
        }

        that.devId = strDeveloperId;
        that.appId = strAppId;
        that.appKey = strAppKey;
        that.appToken = strAppToken;
    }

    /**
     *  配置私有云
     *  @params strAddress      私有云服务地址
     *  @params nPort           私有云服务端口
     *  API说明                   配置私有云信息。当使用私有云时才需要调用该接口配置，默认不需要配置。
     **/
    configServerForPriCloud (strAddress, nPort) {
        var that = this;

        if (typeof strAddress !== "string") {
            that.NotifyError('configServerForPriCloud', 'type of strAddress is not string.');
            return false;
        }

        var ishttps = 'https:' == document.location.protocol ? true : false;
        if (ishttps) {
            if (nPort) {
                that.url = "wss://" + strAddress + ":" + nPort;
            } else {
                that.url = "wss://" + strAddress;
            }
        } else {
            if (nPort) {
                that.url = "ws://" + strAddress + ":" + nPort;
            } else {
                that.url = "ws://" + strAddress;
            }
        }
    }

    /**
     *    加入RTC连线
     *    @params strAnyrtcId     在开发者业务系统中保持唯一标识（理解为房间号）
     *    @params strUserId       主播在开发者自己平台的id（可选）    若不设置发消息接口不能使用
     *    @params strUserData     主播在开发者自己平台的相关信息（昵称，头像等）（可选）       (512字节之内)
     **/
    joinRTCLine (strAnyrtcId, strUserId, strUserData) {
        var that = this;

        if (typeof strAnyrtcId !== "string") {
            /**
             *  创建RTC服务连接结果
             *  nCode           错误码 4  参数非法
             **/
            that.emit('onRTCJoinLineResult', 4);  // 参数非法 
            that.NotifyError('joinRTCLine', 'type of strAnyrtcId is not string.');
            return false;
        }
        that.anyrtcId = strAnyrtcId;

        if (typeof strUserId !== "string") {
            /**
             *  创建RTC服务连接结果
             *  nCode           错误码 4  参数非法
            **/
            that.emit('onRTCJoinLineResult', 4);  // 参数非法
            that.NotifyError('joinRTCLine', 'type of strUserId is not string.');
            return false;
        }
        that.userId = strUserId;

        if (typeof strUserData !== "string") {
            /**
             *  创建RTC服务连接结果
             *  nCode           错误码 4  参数非法
            **/
            that.emit('onRTCJoinLineResult', 4);  // 参数非法
            that.NotifyError('joinRTCLine', 'type of strUserData is not string.');
            return false;
        } else {
            if (that.strlen(strUserData) > 512) {
                /**
                     *  创建RTC服务连接结果
                     *  nCode           错误码 4  参数非法
                **/
                that.emit('onRTCJoinLineResult', 4);  // 参数非法

                that.NotifyError('joinRTCLine', 'strUserData is out of length.');
                return false;
            }
        }
        that.userData = strUserData;

        that.Logcat("Do joinRTC ......");
        that.doInitWSocket();
    }

    /**
     *  申请连麦
     *  @params strBrief     申请连麦请求的消息
     **/
    applyRTCLine (strBrief) {
        var that = this;
        if (that.applyLine == false) {
            that.applyLine = true;
            var objMsg = {};
            objMsg["Cmd"] = "ApplyLine";
            objMsg["Brief"] = strBrief;
            that.doSendMessage(JSON.stringify(objMsg));
        }
    }

    /**
     *  挂断连麦
     **/
    hangupRTCLine () {
        var that = this;
        if (that.applyLine == true) {

            that.applyLine = false;
            /**
            1, 从小程序的房间里面退出来
            */
            var objMsg = {};
            objMsg["Cmd"] = "CancelLine";
            that.doSendMessage(JSON.stringify(objMsg));
        }
    }

    /**
     *  销毁游客端对象
     **/
    clear () {
        var that = this;

        that.events = {};
        /**
        1, 从小程序的房间里面退出来
        */

        if (that.my_id != "") {
            var objMsg = {};
            objMsg["Cmd"] = "Disconnect";
            that.doSendMessage(JSON.stringify(objMsg));

            that.my_id = "";
            that.isAudioLive = false;
        }

        that.doUnInitWSocket();
    }
}

export default RTMPCXcxKit;